//
//  BannerAdView.m
//  fusionsdk
//
//  Created by laole918 on 2020/12/9.
//

#import "BannerAdView.h"
#import <ADSDK/WUBannerADView.h>

@interface BannerAdView()<FlutterStreamHandler, WUBannerAdDelegate>
@property (nonatomic, strong) UIView *containerView;
@property (nonatomic, strong) WUBannerADView *bannerView;
@property (nonatomic, strong) FlutterResult bannerCallback;

@end

@implementation BannerAdView

- (instancetype)initWithFrame:(CGRect)frame
               viewIdentifier:(int64_t)viewId
                    arguments:(id _Nullable)args
                    registrar:(NSObject<FlutterPluginRegistrar> *)registrar
{
    if (self = [super init]) {
        
        // 获取参数
        NSString *codeId;
        CGFloat bannerWidth = 0, bannerHeight = 0;
        if ([args isKindOfClass:[NSDictionary class]]) {
            codeId = args[@"codeId"];
            bannerWidth = [args[@"width"] floatValue];
            bannerHeight = [args[@"height"] floatValue];
        }
        
        if (bannerWidth <= 0.0) {
            bannerWidth = [UIScreen mainScreen].bounds.size.width;
            bannerHeight = bannerWidth /6.4;
        }
        
        if (![codeId isKindOfClass:[NSString class]] || codeId.length == 0) {
            codeId = @"b1";
        }
        
//        CGFloat screenWidth = CGRectGetWidth([UIScreen mainScreen].bounds);
//        CGFloat bannerHeigh = screenWidth/600*90;
        self.bannerView = [[WUBannerADView alloc]initWithSlotID:codeId rootViewController:[[self class] findCurrentShowingViewController] adSize:CGSizeMake(bannerWidth, bannerHeight) IsSupportDeepLink:YES];
        self.bannerView.userInteractionEnabled = YES;
        self.bannerView.frame = CGRectMake(0, 0, bannerWidth, bannerHeight);
        self.bannerView.delegate = self;
        
        // 容器view
        _containerView = [[UIView alloc] initWithFrame:frame];
        _containerView.backgroundColor = [UIColor clearColor];
        
        // 事件通道
        NSString *channelName = [NSString stringWithFormat:@"com.fusion.ad/view_event_%lld", viewId];
        FlutterEventChannel *eventChannel = [FlutterEventChannel eventChannelWithName:channelName binaryMessenger:[registrar messenger]];
        [eventChannel setStreamHandler:self];
    }
    return self;
}
- (nonnull UIView *)view {
    return _containerView;
}

- (FlutterError* _Nullable)onListenWithArguments:(NSString *_Nullable)arguments
                                       eventSink:(FlutterEventSink)events {
    NSLog(@"banner event -> listen");
    if (events) {
        self.bannerCallback = events;
    }
    return nil;
}

- (FlutterError *)onCancelWithArguments:(id)arguments {
    NSLog(@"banner event -> cancel listen");
    return nil;
}

- (void)bannerAdViewDidLoad:(nonnull WUBannerADView *)bannerAdView {
    [self.containerView addSubview:self.bannerView];
    if (self.bannerCallback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdLoad", @"event", nil];
        self.bannerCallback(result);
    }
}

- (void)bannerAdViewDidExposed:(nonnull WUBannerADView *)bannerAdView {
    if (self.bannerCallback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdShow", @"event", nil];
        self.bannerCallback(result);
    }
}

- (void)bannerAdView:(nonnull WUBannerADView *)bannerAdView FailWithError:(NSError * _Nullable)error {
    if (self.bannerCallback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdError", @"event", [NSNumber numberWithInteger:error.code], @"code", [error.userInfo objectForKey:@"NSLocalizedDescription"], @"msg", nil];
        self.bannerCallback(result);
    }
}

- (void)bannerAdViewDidClick:(nonnull WUBannerADView *)bannerAdView {
    if (self.bannerCallback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdClick", @"event", nil];
        self.bannerCallback(result);
    }
}

- (void)bannerAdViewDidClose:(nonnull WUBannerADView *)unifiedBannerView {
    if (self.bannerCallback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdClose", @"event", nil];
        self.bannerCallback(result);
    }
}

// 获取当前显示的 UIViewController
+ (UIViewController *)findCurrentShowingViewController {
    //获得当前活动窗口的根视图
    UIViewController *vc = [UIApplication sharedApplication].keyWindow.rootViewController;
    UIViewController *currentShowingVC = [self findCurrentShowingViewControllerFrom:vc];
    return currentShowingVC;
}

+ (UIViewController *)findCurrentShowingViewControllerFrom:(UIViewController *)vc
{
    // 递归方法 Recursive method
    UIViewController *currentShowingVC;
    if ([vc presentedViewController]) {
        // 当前视图是被presented出来的
        UIViewController *nextRootVC = [vc presentedViewController];
        currentShowingVC = [self findCurrentShowingViewControllerFrom:nextRootVC];

    } else if ([vc isKindOfClass:[UITabBarController class]]) {
        // 根视图为UITabBarController
        UIViewController *nextRootVC = [(UITabBarController *)vc selectedViewController];
        currentShowingVC = [self findCurrentShowingViewControllerFrom:nextRootVC];

    } else if ([vc isKindOfClass:[UINavigationController class]]){
        // 根视图为UINavigationController
        UIViewController *nextRootVC = [(UINavigationController *)vc visibleViewController];
        currentShowingVC = [self findCurrentShowingViewControllerFrom:nextRootVC];

    } else {
        // 根视图为非导航类
        currentShowingVC = vc;
    }

    return currentShowingVC;
}
@end

@interface FusionAdViewFactory()
@property (nonatomic, strong) NSObject<FlutterPluginRegistrar> *registrar;
@end

@implementation FusionAdViewFactory

- (instancetype)initWithRegistrar:(NSObject<FlutterPluginRegistrar> *)registrar {
    self = [super init];
    if (self) {
        _registrar = registrar;
    }
    return self;
}

- (NSObject<FlutterMessageCodec>*)createArgsCodec {
    return [FlutterStandardMessageCodec sharedInstance];
}

- (NSObject<FlutterPlatformView> *)createWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId arguments:(id)args {
    return [[BannerAdView alloc] initWithFrame:frame viewIdentifier:viewId arguments:args registrar:_registrar];
}

@end
